# Hubbard Atom as a Test Case

One may use the Hubbard atom model to benchmark the Feynman diagrams generated by FeynmanDiagram.jl.
In this note, we derive the vertex functions and their power expansion in interaction. The test is implemented in the test folder.

## 1 Hamiltonian
```math
\hat{H}=U n_{\uparrow} n_{\downarrow}-\mu (n_{\uparrow}+n_{\downarrow})
```
where $U$ is the on-site interaction, $\mu$ is the chemical potential.

The eigenstates are,

1. $\left| 0 \right>$ : $E_0=0$.
2. $\left| {\uparrow} \right>$ and $\left| {\downarrow} \right>$ : $E_1=-\mu$.
3. $\left| \uparrow \downarrow \right>$ : $E_2=U-2\mu$.

## 2 Partition sum

```math
Z=1+2e^{\mu\beta}+e^{-\beta(U-2\mu)}
```

## 3 Two-point Green's function

For the imaginary-time $\tau>0$,
```math
G(\tau)=\left<\text{T} c_{\uparrow}(\tau) c^+_{\uparrow}(0) \right>=\frac{1}{Z}\sum_n \left< n\right|e^{-(\beta-\tau)H}c_{\uparrow} e^{-\tau H} c^+_{\uparrow} \left| n \right>
```
Only the states $\left| 0 \right>$ and $\left| {\downarrow} \right>$ contribute,
```math
G(\tau)=\frac{e^{\mu \tau}+e^{\mu \beta}e^{-(U-\mu)\tau}}{Z}
```

For example, at half filling $\mu=U/2$, there is one particle and,

```math
G(\tau)=\frac{e^{-U\tau/2}+e^{-U(\beta-\tau)/2}}{2(1+e^{-U\beta/2})}
```

Transform to the Matsubara frequency,
```math
G(i\omega_n) = \int_0^\beta G(\tau)e^{i\omega_n\tau} d\tau
```
```math
G(i\omega_n)=-\frac{1}{Z}\left(\frac{1+e^{\mu \beta}}{i\omega_n+\mu}+e^{\mu\beta}\frac{1+e^{-(U-\mu)\beta}}{i\omega_n-(U-\mu)}\right)
```
By setting $U=0$, one can show that the above dressed Green's function reduce to the bare one,
```math
g(i\omega_n) = -\frac{1}{i\omega_n+\mu}
```

## 4 Self-energy
The self-energy is defined as $G^{-1}(i\omega_n)=g^{-1}(i\omega_n)-\Sigma(i\omega_n)$, so that
```math
\Sigma(i\omega_n) = \frac{U e^{\beta  \mu } (\mu +i w) \left(e^{\beta  \mu }+e^{\beta  U}\right)}{e^{\beta  U} (-\mu +U-i w)+e^{\beta  (\mu +U)} (-2 \mu +U-2 i w)-e^{2 \beta  \mu } (\mu +i w)}
```

For benchmark purpose, here we also provide the power expansion of the self-energy at the low-energy limit $\omega_0 = \pi /\beta$ with $\mu=0$,
```math
\Sigma(i\omega_0) = -\frac{U}{2}+\frac{(\pi +2 i) \beta  U^2}{8 \pi }-\frac{\left(\pi ^2-4\right) \beta ^2 U^3}{32 \pi ^2}-\frac{\left(24 i-12 \pi +6 i \pi ^2+\pi ^3\right) \beta ^3 U^4}{384 \pi ^3}+\frac{\left(-48-48 i \pi -24 \pi ^2+12 i \pi ^3+5 \pi ^4\right) \beta ^4 U^5}{1536 \pi ^4}+O\left(U^6\right)
```
